%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function description:
% It  returns mean YCbCr values with their standard deviations of the
%estimated illuminant spectra by Color by Correlation method as reported in
%"Color by Correlation: A Simple, Unifying Framework for Color Constancy"
%by G.D. Finlayson, S.D. Hordley and P.M. Hubel, IEEE PAMI, vol. 23, no.
%11, Nov., pp. 1209-1221, 2001
%Input image (CI_ycbcr) is in YCbCr space, The canonical illuminat set is 
% in illuminant_set and the statistics available under ../statdir.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [mY mCb mCr sdY sdCb sdCr max_score]=col_by_corr(CI_ycbcr,illuminant_set,statdir,Y_thresh)
[il_r il_c]=size(illuminant_set); 
[height width depth]=size(CI_ycbcr);

chrom_stat=zeros(32,32,il_c);

% Read statistics

for il=1:il_c
     illuminant=char(illuminant_set(il));
  statfile=[sprintf(statdir) 'stat_' sprintf(illuminant) '.yuv'];
%  sprintf('Opening file %s ', statfile)
  ifp=fopen(statfile,'r');
      dum=fscanf(ifp,'%f',1);
      dum=fscanf(ifp,'%f',1);
      dum=fscanf(ifp,'%f',1);
   
     for l=1:32
         for m=1:32
             chrom_stat(l,m,il)=fscanf(ifp,'%f',1);
         end
     end
     fclose(ifp);
end

   score=zeros(1,il_c);
   chrom_stat_ob=zeros(32,32);

   


 Y=CI_ycbcr(:,:,1);
 Cb=CI_ycbcr(:,:,2);
 Cr=CI_ycbcr(:,:,3);
 

 % collect chromaticity statistics for each object
 for i=1:height
     for j=1:width
          if(Y(i,j)>Y_thresh)
          Cb_val=uint8(floor(Cb(i,j)/8)+1);
          Cr_val=uint8(floor(Cr(i,j)/8)+1);
         chrom_stat_ob(Cb_val,Cr_val)=1;
          end
     end %j
 end %i
 
   % Compute Score
         for l=1:32
             for m=1:32
                 if(chrom_stat_ob(l,m)==1)
                     for til=1:il_c
                     score(til)=score(til)+chrom_stat(l,m,til);
                     end
                 end
             end
         end
     % get maximum illuminants
       [max_score index]=max(score);
%         display(score);
%         display(max_score);
    illuminant=char(illuminant_set(index));
    ref_statfile=[sprintf(statdir) 'stat_' sprintf(illuminant) '.yuvmean'];
    rifp=fopen(ref_statfile,'r');
    mY=fscanf(rifp,'%f',1);
    mCb=fscanf(rifp,'%f',1);
    mCr=fscanf(rifp,'%f',1);
    sdY=fscanf(rifp,'%f',1);
    sdCb=fscanf(rifp,'%f',1);
    sdCr=fscanf(rifp,'%f',1);
    fclose(rifp);
    
    return
      