%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% It demonstrates estimation of illuminant followed by color correction.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;
tic
 
 

 
cm_technique_set={'GRW-DC','MGW-DC','COR-DC','GMAP-DC','NN-DC'}; % Compressed Domain techniques
sp_technique_set={'GRW','MGW','COR','GMAP','NN'}; % Spatial Domain Techniques
cm_correction_set={'DGN-DCT','CSHIFT-DCT'};
sp_correction_set={'DGN-RGB'};

%Color correction set; 
                      
 technique='GMAP-DC';
 color_correction='CSHIFT-DCT';


 
 illuminant_set={'ph-ulm','solux-3500','solux-3500+3202','solux-4100','solux-4100+3202','solux-4700','solux-4700+3202','syl-50mr16q','syl-50mr16q+3202','syl-cwf','syl-wwf'};
 

object='couple';
dst='solux-3500+3202';
imagedir='../test_images/';
statdir='../cconst_stat/';



Y_thresh=1;
 
srcImageFileName=[sprintf(imagedir) sprintf(object) '.bmp'];
targetSPDFilename=[sprintf(statdir) 'stat_' sprintf(dst) '.rgb'];

 if(exist(srcImageFileName,'file')==2)
   src_org_I= imread(srcImageFileName ,'bmp');
else
    sprintf('File %s does not exist.',srcImageFileName)
    return;
end

   ifp=fopen(targetSPDFilename,'r');
   target_R=fscanf(ifp,'%f',1);
   target_G=fscanf(ifp,'%f',1);
   target_B=fscanf(ifp,'%f',1);
   sprintf('target_R=%f target_G=%f target_B=%f',target_R,target_G,target_B)
   [target_Y target_Cb target_Cr]=RGB2YCbCr(target_R,target_G,target_B);
   sprintf('target_Y=%f target_Cb=%f target_Cr=%f',target_Y,target_Cb,target_Cr)



 [s_height s_width s_depth]=size(src_org_I);

figure(1); imshow(uint8(src_org_I)); title(sprintf('%s',object));

  s_CI_ycbcr=double(RGB_YCbCr(src_org_I));
  [dct_s_CI_ycbcr s_maxDC]=get_DCT_JPEG(src_org_I);

  
  s_CI_dc_ycbcr=dct_s_CI_ycbcr(1:8:s_height,1:8:s_width,:)/8;
  s_CI_dc_RGB=double(YCbCr_RGB(s_CI_dc_ycbcr));
  

 

 switch(technique)
     case 'GRW-DC'
           [mY mCb mCr]=mean_illuminant_YCbCr(s_CI_dc_ycbcr,Y_thresh);
           [mR mG mB]=YCbCr2RGB(mY,mCb,mCr);
           kr=target_R/mR; kg=target_G/mG; kb=target_B/mB; 
           shiftU=target_Cb-mCb; shiftV=target_Cr-mCr;
          
           
     case 'MGW-DC'
          [maxR maxG maxB]=max_illuminant_RGB(s_CI_dc_RGB);
          [s_Y s_Cb s_Cr]=RGB2YCbCr(maxR,maxG,maxB);
          kr=target_R/maxR; kg=target_G/maxG; kb=target_B/maxB;
          shiftU=target_Cb-s_Cb; shiftV=target_Cr-s_Cr;
         
     case 'COR-DC'
         [cY  cCb cCr]=col_by_corr(s_CI_dc_ycbcr,illuminant_set,statdir,Y_thresh);
         [cR cG cB]=YCbCr2RGB(cY,cCb,cCr);
         kr=target_R/cR; kg=target_G/cG; kb=target_B/cB;
         shiftU=target_Cb-cCb; shiftV=target_Cr-cCr;
     case 'GMAP-DC'
         [cY  cCb cCr]=col_by_gmap(s_CI_dc_ycbcr,illuminant_set,statdir,Y_thresh);
         [cR cG cB]=YCbCr2RGB(cY,cCb,cCr);
         kr=target_R/cR; kg=target_G/cG; kb=target_B/cB;
         shiftU=target_Cb-cCb; shiftV=target_Cr-cCr;    
    
     case 'NN-DC'
        [mY mCb mCr]=mean_illuminant_YCbCr(s_CI_dc_ycbcr,Y_thresh);
        [cY  cCb cCr]=col_by_NN(mY,mCb,mCr,illuminant_set,statdir);
        [cR cG cB]=YCbCr2RGB(cY,cCb,cCr);
        kr=target_R/cR; kg=target_G/cG; kb=target_B/cB;
        shiftU=target_Cb-cCb; shiftV=target_Cr-cCr;
        
     case 'GRW'
       [s_meanR s_meanG s_meanB]=mean_illuminant_RGB(double(src_org_I),0,0,0);
       kr=target_R/s_meanR; kg=target_G/s_meanG; kb=target_B/s_meanB;
        [s_Y s_Cb s_Cr]=RGB2YCbCr(s_meanR,s_meanG,s_meanB);
        shiftU=target_Cb-s_Cb; shiftV=target_Cr-s_Cr; 
        
     case 'MGW' 
       [s_maxR s_maxG s_maxB]=max_illuminant_RGB(double(src_org_I)); 
       kr=target_R/s_maxR; kg=target_G/s_maxG; kb=target_B/s_maxB;
       [s_Y s_Cb s_Cr]=RGB2YCbCr(s_maxR,s_maxG,s_maxB);
        shiftU=target_Cb-s_Cb; shiftV=target_Cr-s_Cr;  
        
     case 'COR'
         [s_Y s_Cb s_Cr]=col_by_corr(s_CI_ycbcr,illuminant_set,statdir,Y_thresh);
         [s_R s_G s_B]=YCbCr2RGB(s_Y,s_Cb,s_Cr);
         kr=target_R/s_R; kg=target_G/s_G; kb=target_B/s_B;
         shiftU=target_Cb-s_Cb; shiftV=target_Cr-s_Cr;
    
     case 'GMAP'
         [s_Y s_Cb s_Cr]=col_by_gmap(s_CI_ycbcr,illuminant_set,statdir,Y_thresh);
         [s_R s_G s_B]=YCbCr2RGB(s_Y,s_Cb,s_Cr);
         kr=target_R/s_R; kg=target_G/s_G; kb=target_B/s_B;
         shiftU=target_Cb-s_Cb; shiftV=target_Cr-s_Cr;
     
     case 'NN'
          [s_meanR s_meanG s_meanB]=mean_illuminant_RGB(double(src_org_I),0,0,0);
          [s_meanY s_meanCb s_meanCr]=RGB2YCbCr(s_meanR,s_meanG,s_meanB);
          [s_Y s_Cb s_Cr]=col_by_NN(s_meanY,s_meanCb,s_meanCr,illuminant_set,statdir);
          [s_R s_G s_B]=YCbCr2RGB(s_Y,s_Cb,s_Cr);
          kr=target_R/s_R; kg=target_G/s_G; kb=target_B/s_B;
          shiftU=target_Cb-s_Cb; shiftV=target_Cr-s_Cr;
     otherwise
         spritnf('Technique %s not implemented',technique)
 end
 
  switch(color_correction)
      case 'DGN-DCT'
           dct_CI_cc_ycbcr=dct_s_CI_ycbcr;
           for l=0:s_height/8-1
           for m=0:s_width/8-1
            Z=block_diagonal_correction_YCBCR(dct_s_CI_ycbcr(l*8+1:l*8+8,m*8+1:m*8+8,:),kr,kg,kb);
            dct_CI_cc_ycbcr(l*8+1:l*8+8,m*8+1:m*8+8,:)=Z;
           end
           end
           out=get_Image_from_DCT_JPEG(dct_CI_cc_ycbcr);
      case 'CSHIFT-DCT'
          dct_CI_cc_ycbcr=shift_chromaticity_JPEG(dct_s_CI_ycbcr,shiftU,shiftV);
          out=get_Image_from_DCT_JPEG(dct_CI_cc_ycbcr);
      case 'DGN-RGB'
          out=diagonal_correction(double(src_org_I),kr,kg,kb);

      otherwise
          sprintf('correctrion technique %s not implemented',color_correction)
  end

 
  figure(2);  imshow(uint8(out));title(sprintf('%s Technique with Correction %s  to %s',technique,color_correction,dst));
  
  
  toc