%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Function description:
% It desaturates  pixels (w/o saturation).
%I: Input image in YCbCr 
%out: Output image in YCbCr
%ang1,ang2: Apply operations in (ang1,ang2) of hue angles (within 0 to 2*pi)
%s_thresh: Radius of no action around the ref. white (0,0).
%lum_thresh: Luminance Threshold
%wl_factor: factor for White Luminance

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function out=desat_YCbCr(I, ang1,ang2,s_thresh,lum_thresh,wl_factor)

 [height width depth]=size(I);
Y = double(I(:,:,1));
Cb = double(I(:,:,2))-128;
Cr = double(I(:,:,3))-128;

Yw=wl_factor*mean(mean(Y)); %mean(mean( X)) -- avg. of all elements
tY=Y;
tCb=Cb;
tCr=Cr;

	for i=1:height
	    for j=1:width
             val=3.51*Y(i,j)+1.99*Cb(i,j)+0.14*Cr(i,j);
              if(abs(val)>0)
             nCb=Cb(i,j)/val;
             nCr=Cr(i,j)/val;
              else
                  nCb=0;
                  nCr=0;
              end
           
             mag=sqr(nCb)+sqr(nCr);
             if(mag>0) 
                 sat=sqrt(mag);
                 angle=find_angle(nCb,nCr,0,0);
             else
                 sat=0;
                 angle=0;
             end
           if(Y(i,j)>lum_thresh)
            if(sat>s_thresh)
             if((angle>=ang1)&&(angle<ang2))
                wt=Y(i,j)/(Y(i,j)+abs(Yw)); 



                tY(i,j)=abs(Yw)+Y(i,j); 
                [tCb(i,j) tCr(i,j)]=getCbCr(tY(i,j),wt*nCb,wt*nCr);
                if(tY(i,j)<0) 
                    tY(i,j)=0;
                end
                if(tY(i,j)>255) 
                    tY(i,j)=255;
                end
             end
            end
           end
        end
    end
    
 out(:,:,1) = tY;
 out(:,:,2) =  tCb+128;
 out(:,:,3) =  tCr+128;
 
 return