%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Function description:
% It computes maximally saturated pixels.
%I: Input image in YCbCr
%out: Output image in LHS
%ang1,ang2: Apply operations in (ang1,ang2) of hue angles (within 0 to 2*pi)
%s_thresh: Radius of no action around the ref. white (xw,yw).
%(xr,yr), (xg,yg), (xb,yb): Gamut triangle in xy space
%out: in YCbCr space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function out=max_sat_CbCr(I, ang1,ang2,s_thresh,xr,yr,xg,yg,xb,yb,xw,yw)


 [height width depth]=size(I);
Y = double(I(:,:,1));
Cb = double(I(:,:,2))-128.0;
Cr = double(I(:,:,3))-128.0;

out=double(I);
oCb=double(I(:,:,2));
oCr=double(I(:,:,3));
ncount=0;

sprintf('(%f %f), (%f %f),(%f %f)',xr,yr,xg,yg,xb,yb)

	for i=1:height
	    for j=1:width
            val=3.51*Y(i,j)+1.99*Cb(i,j)+0.14*Cr(i,j);
              if(abs(val)>0)
             nCb=Cb(i,j)/val;
             nCr=Cr(i,j)/val;
              else
                  nCb=0;
                  nCr=0;
              end
           
            [x,y]=nCbCr2xy(nCb,nCr);
             mag=sqr(x-xw)+sqr(y-yw);
             if(mag>0) 
                 sat=sqrt(mag);
                 angle=find_angle(x,y,xw,yw);
             else
                 sat=0;
                 angle=0;
             end
           if(sat>s_thresh)
             if((angle>=ang1)&&(angle<ang2))
                 ncount=ncount+1;
               [px,py]=max_sat_xy(angle,xr,yr,xg,yg,xb,yb,xw,yw);
               if((px==0) && (py==0))
                   pnCb=nCb;
                   pnCr=nCr;
               else
               [pnCb,pnCr]=xy2nCbCr(px,py);
               end

                 [oCb(i,j) oCr(i,j)]=getCbCr(Y(i,j),pnCb,pnCr);
                 oCb(i,j)=oCb(i,j)+128;
                 oCr(i,j)=oCr(i,j)+128;
             end
           end
        end
    end
    out(:,:,2)=oCb;
    out(:,:,3)=oCr;
    sprintf('Points modified %d',ncount)
 return;