%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% It demonstrates removal of blocking artifacts.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sprintf('Blocking artifact removal using : Image Filtering with Overlap and Add method')

clear all
tic



mode=0; %0: General 1: Approximation by threshoding a coeff. 2: Sparse Neighbor Computation 3: All Sparse
 Threshold=1e-19; %Threshold for truncating CMP submatrices
 sigma=2.0;
 Blk_size=8;
 Qlt=10; % Quality factor 10
 
 
 
imagedir='../test_images/';
 image = 'boats';
%image = 'watch';

%org_I 		= imread( [sprintf(imagedir) sprintf(image) '.orig.jpg'],'jpg');
org_I 		= imread( [sprintf(imagedir) sprintf(image) '.bmp'],'bmp');
%org_I 		= imread( [sprintf(imagedir) sprintf(image) '.tif'],'tif');
sprintf('Image loaded')


imwrite(org_I,'tmp.jpg','jpg','quality',Qlt);
I= imread('tmp.jpg','jpg');

 
   figure(1); imshow(org_I); title(sprintf(image));
   figure(2); imshow(I); title('Compressed Image');
 
   
   h=truncate_gaussian_1D(3*Blk_size,2*Blk_size,sigma);


    
[nrow flt_length]=size(h);
 if((3*Blk_size+1-flt_length) < 0)
     sprintf('Filter length not appropriate')
     return
 end
    g=[h zeros(1,3*Blk_size+1-flt_length)];
    C_3N=dct_type1_basis(3*Blk_size);
    
     H=C_3N*g';
   
    dct_I=blkproc(I,[Blk_size,Blk_size],@dct2);
      switch(mode)
        case(0)
            sprintf('Performing filtering with exact computation')
        flt_dct_I=blkproc(dct_I,[8 8],[8,8],@filter_overlap_and_add_sym_cosine,H);
        case(1)
            sprintf('Performing filtering with approx. filter coefficients')
         flt_dct_I=blkproc(dct_I,[8 8],[8,8],@approx_filter_overlap_and_add_sym_cosine,H,Threshold);
        case(2)
            sprintf('Performing filtering with sparse neighboring blocks')
            flt_dct_I=blkproc(dct_I,[8 8],[8,8],@sparse_neighbor_filter_overlap_and_add_sym_cosine,H,Threshold);
        case(3)
            sprintf('Performing filtering with all sparse  blocks')
             flt_dct_I=blkproc(dct_I,[8 8],[8,8],@sparse_filter_overlap_and_add_sym_cosine,H,Threshold);
        otherwise
         flt_dct_I=blkproc(dct_I,[8 8],[8,8],@filter_overlap_and_add_sym_cosine,H);
     end
    I_flt=blkproc(flt_dct_I,[Blk_size,Blk_size],@idct2);
     figure(3); imshow(uint8(I_flt)); title('Filtered image in Compression');


gh=[h(flt_length:-1:1) h(2:flt_length)];
gh_2d=convert_separable_2D(gh');;
    
   J=conv2(double(I),gh_2d,'same');
   
   figure(4); imshow(uint8(J)); title('Saptially Convolved image');
   sprintf('Filtered Image PSNR= %g', psnr(I_flt,J))
   
   sprintf(' Compressed  Image PSNR= %g', psnr(I,org_I))
   sprintf('After filtering with original image: PSNR= %g', psnr(I_flt,org_I))
   
toc