%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function description:
% It computes the filtered output from a block by accumulating neighboring
% block's contributions using the FBC.
% X= 24x24 blocks Type-2 DCT coefficients of size 24x24 (with extended
% neighborhood)
% H= 25x1 type-1 DCT coefficients in 1-D
% Y= 8x8 block of type-II DCT coefficients after filtering with H
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=filter_overlap_and_add_sym_cosine(X,H)


[height width]=size(X);

 if(height~=24)
     sprintf('No. of rows should be 24')
     return;
 end
 if(width~=24)
     sprintf('No. of columns should be 24')
     return;
 end
 [filter_length]=size(H);

 if(filter_length~=25)
     sprintf('Filter length (1-D separable) should be 25')
     return;
 end

  A_8_24=block_composition_matrix(3,8);
  D=convert_diagonal_matrix(H(1:24));
  B=A_8_24'*D*A_8_24;
  G=B(1:8,9:16);
  F=B(9:16,9:16);
  E=B(17:24,9:16);
  
   E=sqrt(48)*E;
   F=sqrt(48)*F;
   G=sqrt(48)*G;
   
   
   X11=X(1:8,1:8); X12=X(1:8,9:16); X13=X(1:8,17:24);
   X21=X(9:16,1:8); X22=X(9:16,9:16); X23=X(9:16,17:24);
   X31=X(17:24,1:8); X32=X(17:24,9:16); X33=X(17:24,17:24);
   
   Y1=E*X11*E'+E*X12*F'+E*X13*G';
   Y2=F*X21*E'+F*X22*F'+F*X23*G';
   Y3=G*X31*E'+G*X32*F'+G*X33*G';
  
    Y=(Y1+Y2+Y3);
   
    return