%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description:
% The program demonstrates  filtering with arbitrary separable FIR in the compressed domain with
% the FBC (mode 0), SNC (mode 2) and ASC algorithms (mode 3). It also considers 
% approximation of elements of filtering matrices by thresholding (mode=2).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sprintf('Arbitrary Block Size, Arbitrary type of filter response:  Image Filtering through DCT and DST')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
tic



% approximate_flag=1; % 1 for approximation or  0 for exact
mode=2; %0: General 1: Sparse Neighbor Computation 2: All Sparse
 Threshold=1e-6; %Threshold for truncating CMP submatrices
 sigma=3.0;
 Blk_size=8;
 
imagedir='../test_images/';
 image = 'boats';
%image = 'watch';

%I 		= imread( [sprintf(imagedir) sprintf(image) '.orig.jpg'],'jpg');
I 		= imread( [sprintf(imagedir) sprintf(image) '.bmp'],'bmp');
%I 		= imread( [sprintf(imagedir) sprintf(image) '.tif'],'tif');

   sprintf('Image loaded')

 
   figure(1); imshow(I); title(sprintf(image));
 
   
% Different types of responses. Remove comments to apply a particular type
% of FIR.
   
%  hp=[ 0.267 -0.078 -0.017 0.027] %9/7 lpf analysis
%  hz=0.603;
%  hn=[1.115 -0.591 -0.058 0.091] %9/7 hpf analysis
 
%  hp=[0.591 -0.058 -0.091] %9/7 lpf synthesis
%  hz=0.3;
%  hn=[0.603 -0.267 -0.078 -0.017 0.027] %9/7 hpf synthesis

%  hp=truncate_gaussian_1D(Blk_size/2,Blk_size/2,sigma);
%  hp=[hp(Blk_size/2:-1:2) hp];
%  hz=0;
%  hn=0;
 
 

%  hp=0.5*[6/8 2/8 -1/8] % 5/3 analysis
% hz=1;
%  hn=0.25*[1 -1/2] % 5/3 analysis hpf

% hp=[1/2 1/4] %5/3 synthesis lpf
% hz=1;
% hn=[-2/8 -1/8 -1/16] % 5/3 synthesis hpf

%h=(1/72)*[8 7 6 5 4 3 2 1]; % A LPF by JM

%  hp=0.5*[1 1];
%  hz=0.5;
%  hn=0;

%    hp=(1/9)*[ 1 -2 9 -2 1];
%    hz=0;
%    hn=0;
   
% hp=[1/4 -1/12 1 /12];
% hz=1/2;
% hn=[1/3 -1/6 1/12];

%NC arbitrary
hp=[1/4 -1/12 1/12];
hz=1/2;
hn=[1/3 -1/6 1/12];

%NC Symmetric
% hp=[1/3 -1/6 1/12];
% hz=1/2;
% hn=[1/3 -1/6 1/12];


% hp=[4 -3 -2 -1];
% hz=10;
% hn=[4 -3 -2 -1];

%NC Anti-symmetric
% hp=[1/3 -1/6 1/12];
% hz=0;
% hn=[-1/3 1/6 -1/12];

%Causal
% hp=[1/4 1/8 1/8];
% hz=1/2;
% hn=0;

% hp=[0 1 0 1 2 3];
% hz=3;
% hn=0;
% 
% hp=[4 3 2 1 -1];
% hz=5;
% hn=0;
   
   [nrow plength]=size(hp);
   [nrow nlength]=size(hn);
   
   if(plength>nlength)
       hn=[hn zeros(1,plength-nlength)];
       nlength=plength;
   end
   if(nlength>plength)
       hp=[hp zeros(1,nlength-plength)];
       plength=nlength;
   end

    hs=(hp+hn)/2;
    ha=(hp-hn)/2;

    if((3*Blk_size+1-plength) < 0)
     sprintf('Filter length not appropriate')
     return
    end
    gp=[hz hs zeros(1,3*Blk_size-plength)];
    C_3N=dct_type1_basis(3*Blk_size);
    
     Hp=C_3N*gp';
     
 if((3*Blk_size-1-nlength) < 0)
     sprintf('Filter length not appropriate')
     return
 end
    gn=[ha zeros(1,3*Blk_size-1-nlength)];
    S_3N=dst_type1_basis(3*Blk_size);
    
     Hn=S_3N*gn';
     
    dct_I=blkproc(I,[Blk_size,Blk_size],@dct2);
      switch(mode)
        case(0)
            %corrected on 18/2/2010
            sprintf('Performing filtering with exact computation')
        flt_dct_I=blkproc(dct_I,[Blk_size Blk_size],[Blk_size,Blk_size],@filter_general_arbitrary,Hp,Hn,Blk_size);
        case(1)
            sprintf('Performing filtering with sparse neighboring blocks')
            flt_dct_I=blkproc(dct_I,[Blk_size Blk_size],[Blk_size,Blk_size],@sparse_neighbor_filter_general_arbitrary,Hp,Hn,Blk_size);
        case(2)
            sprintf('Performing filtering with all sparse  blocks')
            flt_dct_I=blkproc(dct_I,[Blk_size Blk_size],[Blk_size,Blk_size],@sparse_filter_general_arbitrary,Hp,Hn,Blk_size);
        otherwise
          flt_dct_I=blkproc(dct_I,[Blk_size Blk_size],[Blk_size,Blk_size],@filter_overlap_and_add_sym_cosine,H);
     end
    I_flt=blkproc(flt_dct_I,[Blk_size,Blk_size],@idct2);
     figure(2); imshow(uint8(I_flt)); title('Filtered image in Compression');


 ghs=[hs(plength:-1:1) hz hs(1:plength)];
 gha=[-ha(nlength:-1:1) 0 ha(1:nlength)];

gh=[hn(nlength:-1:1) hz hp(1:plength)];
gh_2d=convert_separable_2D(gh');;
    
   J=conv2(double(I),gh_2d,'same');
   
   figure(3); imshow(uint8(J)); title('Convolved image');
   sprintf('Filtered Image PSNR= %g', psnr(I_flt,J))
     
toc