%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description:
% The program demonstrates Gaussian filtering (symmetric FIR) in the compressed domain with
% the FBC (mode 0), SNC (mode 2) and ASC algorithms (mode 3). It also considers 
% approximation of elements of filtering matrices by thresholding (mode=2).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sprintf('Image Filtering with Overlap and Add method')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
tic

 mode=3; %0: General 1: Approximation by threshoding a coeff. 2: Sparse Neighbor Computation 3: All Sparse
 Threshold=1e-6; %Threshold for truncating CMP submatrices
 sigma=5.0;
 
 imagedir='../test_images/';
 image = 'boats';
%image = 'watch';

%I 		= imread( [sprintf(imagedir) sprintf(image) '.orig.jpg'],'jpg');
I 		= imread( [sprintf(imagedir) sprintf(image) '.bmp'],'bmp');
%I 		= imread( [sprintf(imagedir) sprintf(image) '.tif'],'tif');
sprintf('Image loaded')
 
   figure(1); imshow(I); title(sprintf(image));
   
   h=truncate_gaussian_1D(25,8,sigma);
   
    
    C_24=dct_type1_basis(24);
    
     H=C_24*h';
   
    dct_I=blkproc(I,[8,8],@dct2);
      switch(mode)
        case(0)
            sprintf('Performing filtering with exact computation')
        flt_dct_I=blkproc(dct_I,[8 8],[8,8],@filter_overlap_and_add_sym_cosine,H);
        case(1)
            sprintf('Performing filtering with approx. filter coefficients')
        flt_dct_I=blkproc(dct_I,[8 8],[8,8],@approx_filter_overlap_and_add_sym_cosine,H,Threshold);
        case(2)
            sprintf('Performing filtering with sparse neighboring blocks')
            flt_dct_I=blkproc(dct_I,[8 8],[8,8],@sparse_neighbor_filter_overlap_and_add_sym_cosine,H,Threshold);
        case(3)
            sprintf('Performing filtering with all sparse  blocks')
            flt_dct_I=blkproc(dct_I,[8 8],[8,8],@sparse_filter_overlap_and_add_sym_cosine,H,Threshold);
        otherwise
          flt_dct_I=blkproc(dct_I,[8 8],[8,8],@filter_overlap_and_add_sym_cosine,H);
     end
    I_flt=blkproc(flt_dct_I,[8,8],@idct2);
    figure(2); imshow(uint8(I_flt)); title('Filtered image in Compression');

    gh=gaussian(17,17,9,9,sigma);
    
   J=conv2(double(I),gh,'same');
   
   figure(3); imshow(uint8(J)); title('Convolved image');
   sprintf('Filtered Image PSNR= %g', psnr(I_flt,J))
   
toc
