%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function description:
% It computes the filtered output for any arbitrary FIR from a block by accumulating neighboring
% block's contributions using the SNC. Neigboring filtering matrices are made sparse by considering
% their N/2xN/2 elements only.
% X= 3Nx3N blocks Type-2 DCT coefficients of size 3Nx3N (with extended
% neighborhood)
% Hp= (3N+1)x1 Type-1 DCT coefficients in 1-D
% Hn=(3N-1)x1 Type-1 DST coefficents in 1-D
% Y= NxN block of DCT-II coefficients after filtering with H
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=sparse_neighbor_filter_general_arbitrary(X,Hp,Hn,N)

[height width]=size(X);

 L=3*N;
 if(height~=L)
     sprintf('No. of rows should be %d',L)
     sprintf('It is %d ',height)
     return;
 end
 if(width~=L)
     sprintf('No. of columns should be %d',L)
     sprintf('It is %d ',width)
     return;
 end
 [plength]=size(Hp);

 if(plength~=3*N+1)
     sprintf('Filter length (1-D separable) should be %d', 3*N+1)
     return;
 end

  [nlength]=size(Hn);

 if(nlength~=3*N-1)
     sprintf('Filter length (1-D separable) should be %d', 3*N-1)
     return;
 end

 
 
  A=block_composition_matrix(3,N);
  Dp=convert_diagonal_matrix(Hp(1:3*N));
   Bp=A'*Dp*A;
  Gp=Bp(1:N,N+1:2*N);
  Fp=Bp(N+1:2*N,N+1:2*N);
  Ep=Bp(2*N+1:3*N,N+1:2*N);
  
 
SC=convert_CST(3,N);
Bn=A'*convert_off_diagonal_matrix(Hn,-1)*SC;



 
  Gn=Bn(1:N,N+1:2*N);
  Fn=Bn(N+1:2*N,N+1:2*N);
  En=Bn(2*N+1:3*N,N+1:2*N);
  
  
   
   Ep=sqrt(6*N)*Ep;
   Fp=sqrt(6*N)*Fp;
   Gp=sqrt(6*N)*Gp;
   
   En=sqrt(6*N)*En;
   Fn=sqrt(6*N)*Fn;
   Gn=sqrt(6*N)*Gn;
   
    E=Ep-En;
    F=Fp-Fn;
    G=Gp-Gn;
    
   X11=zeros(N); X12=zeros(N); X13=zeros(N);
   X21=zeros(N); X22=zeros(N); X23=zeros(N);
   X31=zeros(N); X32=zeros(N); X33=zeros(N);
   
   X11(1:N/2,1:N/2)=X(1:N/2,1:N/2); X12(1:N/2,1:N/2)=X(1:N/2,N+1:3*N/2); X13(1:N/2,1:N/2)=X(1:N/2,2*N+1:5*N/2);
   X21(1:N/2,1:N/2)=X(N+1:3*N/2,1:N/2); X22(1:N,1:N)=X(N+1:2*N,N+1:2*N); X23(1:N/2,1:N/2)=X(N+1:3*N/2,2*N+1:5*N/2);
   X31(1:N/2,1:N/2)=X(2*N+1:5*N/2,1:N/2); X32(1:N/2,1:N/2)=X(2*N+1:5*N/2,N+1:3*N/2); X33(1:N/2,1:N/2)=X(2*N+1:5*N/2,2*N+1:5*N/2);
   

    Y1=E*X11*E'+E*X12*F'+E*X13*G';
    Y2=F*X21*E'+F*X22*F'+F*X23*G';
    Y3=G*X31*E'+G*X32*F'+G*X33*G';
   
     Y=Y1+Y2+Y3;
    return