%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function description:
% It computes the filtered output from a block by accumulating neighboring
% block's contributions using the SNC. Neighboring filtering matrices are made sparse by considering
% their N/2 x N/2 elements only.
% X= 3Nx3N blocks Type-2 DCT coefficients of size 24x24 (with extended
% neighborhood)
% H= (3N+1)x1 type-1 DCT coefficients in 1-D
% Y= NxN block of type-II DCT coefficients after filtering with H
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=sparse_neighbor_filter_overlap_and_add_arbitrary(X,H,N,thresh)


[height width]=size(X);

 L=3*N;
 if(height~=L)
     sprintf('No. of rows should be %d',L)
     sprintf('It is %d ',height)
     return;
 end
 if(width~=L)
     sprintf('No. of columns should be %d',L)
     sprintf('It is %d ',width)
     return;
 end
 [filter_length]=size(H);

 if(filter_length~=3*N+1)
     sprintf('Filter length (1-D separable) should be %d', 3*N+1)
     return;
 end

 
  A_3_N=block_composition_matrix(3,N);
  D=convert_diagonal_matrix(H(1:3*N));
   B=A_3_N'*D*A_3_N;
  G=B(1:N,N+1:2*N);
  F=B(N+1:2*N,N+1:2*N);
  E=B(2*N+1:3*N,N+1:2*N);
  
   E=sqrt(6*N)*E;
   F=sqrt(6*N)*F;
   G=sqrt(6*N)*G;
   
   E=truncate_matrix(E,thresh);
   F=truncate_matrix(F,thresh);
   G=truncate_matrix(G,thresh);
   
    X11=zeros(N); X12=zeros(N); X13=zeros(N);
   X21=zeros(N); X22=zeros(N); X23=zeros(N);
   X31=zeros(N); X32=zeros(N); X33=zeros(N);
   
   X11(1:N/2,1:N/2)=X(1:N/2,1:N/2); X12(1:N/2,1:N/2)=X(1:N/2,N+1:3*N/2); X13(1:N/2,1:N/2)=X(1:N/2,2*N+1:5*N/2);
   X21(1:N/2,1:N/2)=X(N+1:3*N/2,1:N/2); X22(1:N,1:N)=X(N+1:2*N,N+1:2*N); X23(1:N/2,1:N/2)=X(N+1:3*N/2,2*N+1:5*N/2);
   X31(1:N/2,1:N/2)=X(2*N+1:5*N/2,1:N/2); X32(1:N/2,1:N/2)=X(2*N+1:5*N/2,N+1:3*N/2); X33(1:N/2,1:N/2)=X(2*N+1:5*N/2,2*N+1:5*N/2);
   
   
   Y1=E*X11*E'+E*X12*F'+E*X13*G';
   Y2=F*X21*E'+F*X22*F'+F*X23*G';
   Y3=G*X31*E'+G*X32*F'+G*X33*G';
  
    Y=(Y1+Y2+Y3);
  

   
    return