%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function Description:
% It downsamples a block by a factor of 2 using the linear and distributive property of DCT.
%  X : 16x16 elements i.e. 4 8x8 DCT blocks. 
%  Y : 8 X 8 block.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=LDU_downsize(X)



[height width]=size(X);

if((height~=16) || (width~=16))
    sprintf('Implemented only for 8x8 blocks');
    return
end

 B11=X(1:height/2,1:width/2);
 B12=X(1:height/2,width/2+1:width);
 B21=X(height/2+1:height,1:width/2);
 B22=X(height/2+1:height,width/2+1:width);
 

 Z_4_8=zeros(4,8);
 D_4_8=[0.5 0.5  0   0   0   0   0  0;
         0   0  0.5 0.5  0   0   0  0;
         0   0   0   0  0.5 0.5  0  0;
         0   0   0   0   0   0  0.5 0.5;];
     
         p_1=[D_4_8;Z_4_8];
         p_2=[Z_4_8; D_4_8];
         
         P1=dct2(p_1);
         P2=dct2(p_2);
         
         
         Y=P1*B11*P1'+P1*B12*P2'+P2*B21*P1'+P2*B22*P2';
 
  return