%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function Description:
% It downsamples a block by a factor of 2 using a CMP.
%  X : 16x16 elements i.e. 4 8x8 DCT blocks.
%  Y : 8x8 DCT block.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=Park_downsize(X)
% Applicable only for 8x8 block size

[height width]=size(X);

 if((height~=16) || (width ~=16))
     sprintf('Only valid for 16x16 blocks')
     return
 end
 
 

 W=[16 10.2023 0 -3.4449 0 2.1214 0 -1.5763 0 1.2936 0 -1.1339 0 1.0450 0 -1.0048];

 

 
 W_d0=[W(1)      0        0        0         0           0           0          0 ;
      W(2) W(2)+W(4) W(4)+W(6) W(6)+W(8) W(8)+W(10) W(10)+W(12) W(12)+W(14) W(14)+W(16);
      0      W(1)       0        0         0           0           0          0 ;
      W(4) W(2)+W(6) W(2)+W(8) W(4)+W(10) W(6)+W(12) W(8)+W(14) W(10)+W(16) W(12)-W(16);
      0      0       W(1)    0         0           0               0          0 ;
      W(6) W(4)+W(8) W(2)+W(10) W(2)+W(12) W(4)+W(14) W(6)+W(16) W(8)-W(16) W(10)-W(14);
       0      0       0           W(1)      0           0           0          0 ;
      W(8) W(6)+W(10) W(4)+W(12) W(2)+W(14) W(2)+W(16) W(4)-W(16) W(6)-W(14) W(8)-W(12)];
  
  S=[1  0   0   0  0 0  0 0 ;
     0  -1  0   0  0 0  0 0 ;
     0   0  1   0  0 0  0 0 ;
     0   0  0  -1  0 0  0 0 ;
     0   0  0   0  1 0  0 0 ;
     0   0  0   0  0 -1 0 0 ;
     0   0  0   0  0 0  1  0 ;
     0   0  0   0  0 0  0 -1 ];

 
  X11=X(1:8,1:8);
  X12=X(1:8,9:16);
  X21=X(9:16,1:8);
  X22=X(9:16,9:16);
  
  F=[4*sqrt(2)  0   0   0  0 0  0 0 ;
     0  4  0   0  0 0  0 0 ;
     0   0 4   0  0 0  0 0 ;
     0   0  0  4  0 0  0 0 ;
     0   0  0   0  4 0  0 0 ;
     0   0  0   0  0 4 0 0 ;
     0   0  0   0  0 0  4  0 ;
     0   0  0   0  0 0  0 4 ];
 G=inv(F);
 Q=G*W_d0*F;
 P=S*Q*S;
 Y=Q*X11*Q'+Q*X12*P'+P*X21*Q'+P*X22*P';
 Y=Y/1024;

  return