%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function Description:
% Filtering in block DCT domain with separable symmetric filter.
% X= NxN blocks Type-2 DCT coefficients of size 3Nx3N (with extended
% neighborhood)
% El,Fl,Gl= NxN filtering matrices: left operands.
% Er,Fr,Gr= NxN filtering matrices: right operands.
% Y= NxN block of DCT-II coefficients after filtering.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=filter_overlap_and_add_separable_arbitrary(X,El,Fl,Gl,Er,Fr,Gr,N)


[height width]=size(X);

 L=3*N;
 if(height~=L)
     sprintf('No. of rows should be %d',L)
     sprintf('It is %d ',height)
     return;
 end
 if(width~=L)
     sprintf('No. of columns should be %d',L)
     sprintf('It is %d ',width)
     return;
 end
 
  
   
   X11=X(1:N,1:N); X12=X(1:N,N+1:2*N); X13=X(1:N,2*N+1:3*N);
   X21=X(N+1:2*N,1:N); X22=X(N+1:2*N,N+1:2*N); X23=X(N+1:2*N,2*N+1:3*N);
   X31=X(2*N+1:3*N,1:N); X32=X(2*N+1:3*N,N+1:2*N); X33=X(2*N+1:3*N,2*N+1:3*N);
   
   Y1=El*X11*Er'+El*X12*Fr'+El*X13*Gr';
   Y2=Fl*X21*Er'+Fl*X22*Fr'+Fl*X23*Gr';
   Y3=Gl*X31*Er'+Gl*X32*Fr'+Gl*X33*Gr';
  
  % Y=(2*24)*(Y1+Y2+Y3);
    Y=(Y1+Y2+Y3);
  

   
    return