%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description:
% It demonstrates transcoding from DWT to DCT.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sprintf('Wavelet to DCT Transcoding: version 2 18/9/10')
 
close all
clf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
tic

Blk_size=4;
fig_count=0;

Be=dct_up2_bs_even(Blk_size);
Bo=dct_up2_bs_odd(Blk_size);

imagedir='../test_images/';
image = 'watch.orig';



 I 		= imread( [sprintf(imagedir) sprintf(image) '.jpg'],'jpg');
 sprintf('Image loaded')
 
dI=double(I);
 
 [height width]=size(I);
   
   fig_count=fig_count+1; figure(fig_count); 
   imshow(I); title(sprintf(image));
   
ha=[6/8 2/8 -1/8] % 5/3 analysis
ga=[1 -1/2] % 5/3 analysis hpf

hs=[1 1/2] %5/3 synthesis lpf
gs=[6/8 -2/8 -1/8] % 5/3 synthesis hpf

%     ha=[0.603 0.267 -0.078 -0.017 0.027] %9/7 lpf analysis
%     ga=[1.115 -0.591 -0.058 0.091] %9/7 hpf analysis
%     
%    hs=[1.115 0.591 -0.058 -0.091] %9/7 lpf synthesis
%    gs=[0.603 -0.267 -0.078 0.017 0.027] %9/7 hpf synthesis
    




   [nrow hlength]=size(ha);
   [nrow glength]=size(ga);
    
    
f_ha=[ha(hlength:-1:2)  ha(1:hlength)];
f_ga=[ga(glength:-1:2)  ga(1:glength)];

% Obtain 1st level DWT of the image
 [CA,CH,CV,CD]=dwtImage(I,f_ha,f_ga); % CA, CH, CV, CD are wavelet sub-bands
   
   


   fig_count=fig_count+1; figure(fig_count); 
   imshow(uint8(CA)); title('CA');
   
   fig_count=fig_count+1; figure(fig_count);  
   
   imshow(uint8(CH));title('CH');
   
   fig_count=fig_count+1; figure(fig_count); 
   
   imshow(uint8(CV));title('CV');
   
   fig_count=fig_count+1; figure(fig_count); 
  
   imshow(uint8(CD)); title('CD');
   
      
        dct_CA=blkproc(CA,[Blk_size,Blk_size],@upsample_DCT,Be,Be);
        dct_CH=blkproc(CH,[Blk_size,Blk_size],@upsample_DCT,Be,Bo);
        dct_CV=blkproc(CV,[Blk_size,Blk_size],@upsample_DCT,Bo,Be);
        dct_CD=blkproc(CD,[Blk_size,Blk_size],@upsample_DCT,Bo,Bo);
        
   
   [nrow hlength]=size(hs);
   [nrow glength]=size(gs);
    
    
f_hs=[hs(hlength:-1:2)  hs(1:hlength)];
f_gs=[gs(glength:-1:2)  gs(1:glength)];


     % Check the reconstruction through IDWT
     J=idwtImage(CA,CH,CV,CD,f_hs,f_gs);
    fig_count=fig_count+1; figure(fig_count); 
    imshow(uint8(J)); title('Reconstructed Image in spatial domain');
    sprintf('Spatial PSNR=%f DB',psnr(I(8:height-8,8:width-8),J(8:height-8,8:width-8)))   

    % Transcoding into DCT blocks
    
    [nrow flt_length]=size(hs);
     if((3*2*Blk_size+1-flt_length) < 0)
         sprintf('Filter length not appropriate')
         return
     end
    he=[hs zeros(1,3*2*Blk_size+1-flt_length)];
    C_3N=dct_type1_basis(3*2*Blk_size);
    
     Hl=C_3N*he';
     
     [nrow flt_length]=size(gs);
     if((3*2*Blk_size+1-flt_length) < 0)
         sprintf('Filter length not appropriate')
         return
     end
    ge=[gs zeros(1,3*2*Blk_size+1-flt_length)];
   
    
     Hr=C_3N*ge';
     
     N=2*Blk_size;
     A_3_N=block_composition_matrix(3,N);
 
  D=convert_diagonal_matrix(Hl(1:3*N));
   B=A_3_N'*D*A_3_N;
  Gl=sqrt(6*N)*B(1:N,N+1:2*N);
  Fl=sqrt(6*N)*B(N+1:2*N,N+1:2*N);
  El=sqrt(6*N)*B(2*N+1:3*N,N+1:2*N);
  
   
   
  D=convert_diagonal_matrix(Hr(1:3*N));
  B=A_3_N'*D*A_3_N;
  Gr=sqrt(6*N)*B(1:N,N+1:2*N);
  Fr=sqrt(6*N)*B(N+1:2*N,N+1:2*N);
  Er=sqrt(6*N)*B(2*N+1:3*N,N+1:2*N);
  
   
     
      flt_dct_CA=blkproc(dct_CA,[2*Blk_size 2*Blk_size],[2*Blk_size,2*Blk_size],@filter_overlap_and_add_separable_arbitrary,El,Fl,Gl,El,Fl,Gl,2*Blk_size);
      flt_dct_CH=blkproc(dct_CH,[2*Blk_size 2*Blk_size],[2*Blk_size,2*Blk_size],@filter_overlap_and_add_separable_arbitrary,El,Fl,Gl,Er,Fr,Gr,2*Blk_size);
      flt_dct_CV=blkproc(dct_CV,[2*Blk_size 2*Blk_size],[2*Blk_size,2*Blk_size],@filter_overlap_and_add_separable_arbitrary,Er,Fr,Gr,El,Fl,Gl,2*Blk_size);
      flt_dct_CD=blkproc(dct_CD,[2*Blk_size 2*Blk_size],[2*Blk_size,2*Blk_size],@filter_overlap_and_add_separable_arbitrary,Er,Fr,Gr,Er,Fr,Gr,2*Blk_size);
     
    
      flt_I=flt_dct_CA+flt_dct_CH+flt_dct_CV+flt_dct_CD; %flt_I is the transcoded image in block DCT space
   
    I_synth=blkproc(flt_I,[2*Blk_size,2*Blk_size],@idct2);
    fig_count=fig_count+1; figure(fig_count); 
    imshow(uint8(I_synth)); title('Synthesized IDCT of  Image');
   
    sprintf('PSNR=%f DB',psnr(I(8:height-8,8:width-8),I_synth(8:height-8,8:width-8)))
toc