%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These codes are written by Prof. Jayanta Mukhopadhyay, IIT Kharagpur.
% The codes may be used freely for any academic and research 
% purpose with appropriate acknowledgement.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function description:
% Converts pixels in  normlaized CIE-XYZ (NXYZ) color space to 
% LHS (Intensity-hue-saturation according to CIE Chromaticity chart). 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out=NXYZ_to_LHS(I,xw,yw)
%I: input image in NXYZ space
% (xw,yw): white reference, usually (0.32,0.33)

[height width depth]=size(I);
nx = double(I(:,:,1));
ny = double(I(:,:,2));
nz = double(I(:,:,3));

lum=zeros(height,width);
angle=zeros(height,width);
sat=zeros(height,width);
        
	for i=1:height
	    for j=1:width
              sat(i,j)=sqr(nx(i,j)-xw)+sqr(ny(i,j)-yw);
		      if(sat(i,j)>0) 
                  sat(i,j)=sqrt(sat(i,j));
              end
		      angle(i,j)= find_angle(nx(i,j),ny(i,j),xw,yw);
              lum(i,j)=nz(i,j);
        end
    end
                
out(:,:,1) = lum;
out(:,:,2) = angle;
out(:,:,3) = sat;
	return